use std::time::Duration;

use crate::cli::{GivenDay, GivenTask};

use super::FileToBenchmark;

#[derive(Debug)]
pub struct BenchmarkResult {
    task: GivenTask,
    day: GivenDay,
    how_long: Duration,
    maybe_different_actual_output: Option<String>,
    expected_output: String,
}

impl BenchmarkResult {
    pub fn new(benchmarking: FileToBenchmark, actual_ouput: String, how_long: Duration) -> Self {
        let (day, task, expected_output) = benchmarking.into();
        let maybe_different_actual_output = if actual_ouput == expected_output {
            None
        } else {
            Some(actual_ouput)
        };
        Self {
            task,
            day,
            how_long,
            maybe_different_actual_output,
            expected_output,
        }
    }

    pub fn matched_with_expected_output(&self) -> bool {
        self.maybe_different_actual_output().is_none()
    }

    pub fn task(&self) -> GivenTask {
        self.task
    }

    pub fn day(&self) -> GivenDay {
        self.day
    }

    pub fn how_long_as_string(&self) -> String {
        Self::convert_duration_to_secs_and_mili_txt(self.how_long)
    }

    pub fn maybe_different_actual_output(&self) -> Option<&str> {
        self.maybe_different_actual_output.as_deref()
    }

    pub fn expected_output(&self) -> &str {
        &self.expected_output
    }

    pub fn how_long(&self) -> Duration {
        self.how_long
    }

    pub fn convert_duration_to_secs_and_mili_txt(how_long: Duration) -> String {
        let secs = how_long.as_secs();
        let mili_secs = how_long.as_nanos() % 1_000_000_000;
        format!("{}:{}", secs, mili_secs)
    }
}
